"""Test request related code."""

import pytest


@pytest.mark.parametrize(
    "kwargs, expected_locale",
    (
        (  # not filled __LOCALE__ , should return default one
            {"slug": "some-slug"},
            "en",
        ),
        (  # filled __LOCALE__ within available, returned exact that
            {"slug": "some-slug", "__LOCALE__": "pl"},
            "pl",
        ),
        (  # filled __LOCALE__ within one not in available, returned default one
            {"slug": "some-slug", "__LOCALE__": "fr"},
            "en",
        ),
    ),
)
def test_request(web_request, kwargs, expected_locale):
    """Test whether route-parameters gets filled correctly."""
    route_params = web_request.default_locale(**kwargs)
    assert "__LOCALE__" in route_params
    assert route_params["__LOCALE__"] in web_request.registry["localize"]["locales"]["available"]
    assert route_params["__LOCALE__"] == expected_locale


def test_locale_id(db_locales, web_request):
    """Test for creating, and getting loacel id."""
    assert isinstance(web_request.locale_id, int)


def test_locales(db_locales, web_request):
    """Test return locales list."""
    assert len(web_request.locales()) == 3  # noqa: PLR2004


def test_locales_config(db_locales, web_request):
    """Test return locales list limited by config.

    There's a new locale, so it should create new Language entry.
    """
    assert len(web_request.locales(True)) == 4  # noqa: PLR2004



"""Tools related tests."""

import os
import sys

import pytest
from mock import Mock
from pyramid.i18n import Localizer
from pyramid.interfaces import ILocalizer
from pyramid.path import package_path

from pyramid_localize import build_localize_config
from pyramid_localize.tools import destination_path, dummy_autotranslate, set_localizer
from tests.conftest import web_request_func


def test_simple(web_request):
    """Simple localizer setting test on a request."""
    set_localizer(web_request)
    assert isinstance(web_request.localizer, Localizer)
    assert hasattr(web_request, "_")


def test_reset(web_request):
    """Test resetting localizer capabilites.

    1. localizer gets set on a request.
    2. Reset localizer call gets issued.
    3. New request have new localizer set.
    """
    # set (and create localizer)
    set_localizer(web_request)
    old_localizer = web_request.localizer

    queried_localizer = web_request.registry.queryUtility(ILocalizer, name=web_request.locale_name)
    assert queried_localizer == web_request.localizer
    # resetting localizer
    set_localizer(web_request, reset=True)
    # these are equal due to request.localizer
    # being reify property since pyramid 1.5
    assert old_localizer == web_request.localizer
    queried_localizer_after_reset = web_request.registry.queryUtility(ILocalizer, name=web_request.locale_name)

    assert queried_localizer_after_reset is not queried_localizer
    # let's create a new request, to check that
    request = web_request_func()
    set_localizer(web_request)
    assert old_localizer is not request.localizer


def test_translate(web_request):
    """Simple test for translating method call."""
    msgid = "Test message"
    set_localizer(web_request)
    assert msgid == web_request._(msgid)


def test_destination_filename():
    """Testing translation fullpath resolve."""
    request = Mock()
    path = "/some/path/to/translations"
    request.registry = {"localize": build_localize_config({"localize.translation.destination": path})}
    result = destination_path(request)
    assert result == path


def test_destination_package():
    """Testing translation package:path resolve."""
    request = Mock()
    request.registry = {"localize": build_localize_config({"localize.translation.destination": "tests:translations"})}
    result = destination_path(request)
    assert result == os.path.join(package_path(sys.modules["tests"]), "translations")


@pytest.mark.parametrize(
    "kwargs, result",
    (
        ({"msgid": "Simple fake text"}, "Simple fake text"),
        ({"msgid": "test-msgid", "default": "Simple fake text"}, "Simple fake text"),
        (
            {"msgid": "Simple ${what} text", "mapping": {"what": "fake"}},
            "Simple fake text",
        ),
    ),
)
def test_dummy_message(kwargs, result):
    """Test dummy autotranslate method."""
    assert dummy_autotranslate(**kwargs) == result



# Copyright (c) 2013-2014 by pyramid_localize authors and contributors <see AUTHORS file>
#
# This module is part of pyramid_localize and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""View submodules."""



# Copyright (c) 2013-2014 by pyramid_localize authors and contributors <see AUTHORS file>
#
# This module is part of pyramid_localize and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""i18n subscribers."""

from pyramid.events import BeforeRender, NewRequest, subscriber

from pyramid_localize.tools import set_localizer


@subscriber(BeforeRender)
def global_renderer(event):
    """Add localizer, and translation methods to context."""
    request = event["request"]
    set_localizer(request)

    event["_"] = request._
    event["localizer"] = request.localizer


@subscriber(NewRequest)
def add_localizer(event):
    """Add localizer and translation methods to request."""
    set_localizer(event.request)



# Copyright (c) 2013-2014 by pyramid_localize authors and contributors <see AUTHORS file>
#
# This module is part of pyramid_localize and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""Localize route predicate."""


def language(field):
    """Create language predicate for given url match field."""

    def predicate(info, request):
        """Check whether language is one of the defaults."""
        if field in info["match"] and info["match"][field] in request.registry["localize"]["locales"]["available"]:
            return True
        return False

    return predicate


language.__text__ = "language predicate, to determine allowed languages in route"



# Copyright (c) 2013-2014 by pyramid_localize authors and contributors <see AUTHORS file>
#
# This module is part of pyramid_localize and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""Language model."""
import gettext

import pycountry
from pyramid_basemodel import Base
from sqlalchemy import Column, DateTime, Integer, Sequence, String, Unicode, event, func


class Language(Base):
    """Language table model definition."""

    __tablename__ = "languages"

    id = Column(Integer, Sequence(__tablename__ + "_sq"), primary_key=True)
    updated_at = Column(
        DateTime,
        default=func.now(),
        onupdate=func.now(),
        nullable=False,
    )
    name = Column(Unicode(45), nullable=False)
    native_name = Column(Unicode(45), nullable=False)
    language_code = Column(String(2), unique=True, nullable=False)  # ISO 639-1 (Alpha2)

    def __unicode__(self):  # pragma: no cover
        """Language to unicode conversion."""
        return self.name

    def __str__(self):  # pragma: no cover
        """Language to string conversion."""
        return self.name.encode("utf8")


@event.listens_for(Language, "before_insert")
def before_language_insert(_, __, language):
    """Set name and native_name before creation."""
    # Check language code
    lang_data = pycountry.languages.get(alpha_2=language.language_code)
    if lang_data is None:
        # Language code not recognized, set defaults
        language.name = "UNKNOWN"
        language.native_name = "UNKNOWN"
        return

    # Set name and native_name
    language.name = str(lang_data.name)

    if language.language_code == "en":
        # English does not have a translation file
        language.native_name = str(lang_data.name)

    else:
        lang_locale = gettext.translation("iso639-3", pycountry.LOCALES_DIR, languages=[language.language_code])
        localize = lang_locale.gettext

        language.native_name = str(localize(lang_data.name))



# Copyright (c) 2013-2014 by pyramid_localize authors and contributors <see AUTHORS file>
#
# This module is part of pyramid_localize and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""Catalogue view."""


import logging
import os
import subprocess
import sys
import time

from pyramid.asset import resolve_asset_spec
from pyramid.httpexceptions import HTTPFound
from pyramid.path import package_path
from pyramid.view import view_config, view_defaults

from pyramid_localize.tools import destination_path, set_localizer

log = logging.getLogger(__name__)


@view_defaults(
    permission="manage_translations",
    renderer="pyramid_localize:resources/templates/index.mako",
)
class CatalogueView(object):
    """View class for catalogue manipulation actions."""

    def __init__(self, request):
        """Assign request.

        :param pyramid.request.Request request:
        """
        self.request = request

    def _translation_file(self, language, domain, extension="po"):
        """Create a translation file path.

        :param str language: two-letter language code
        :param str domain: translation domain name
        :param str extension: translation file extension (po/mo)
        """
        translation_destination = destination_path(self.request)
        return os.path.abspath(
            os.path.join(
                translation_destination,
                language,
                "LC_MESSAGES",
                domain + "." + extension,
            )
        )

    def _translation_template_path(self, spec):
        """Calculate path to translation template file.

        :param str spec: either full path, or package related path
        """
        # resolving possible asset spec to path (pyramid way):
        package_name, filename = resolve_asset_spec(spec)
        if package_name is None:  # absolute filename
            return os.path.abspath(filename)
        __import__(package_name)
        package = sys.modules[package_name]
        return os.path.abspath(os.path.join(package_path(package), filename))

    @view_config(route_name="localize:index")
    def index(self):
        """List domains, and its files of files with metadata.

        :returns:

            .. code-block:: python

                {
                    'language': {
                        'domain1': {
                            'po': 'modification time',
                            'pot': 'modification time',
                            'mo': 'modification time',
                            },
                        # more domains
                    },
                    # more languages
                }
        """
        translations = {}
        translation_sources = self.request.registry["localize"]["translation"]["sources"]

        for language in self.request.registry["localize"]["locales"]["available"]:
            translations[language] = {}
            for domain in translation_sources:
                translations[language][domain] = {"po": None, "pot": None, "mo": None}
                po_file = self._translation_file(language, domain)
                if os.path.isfile(po_file):
                    translations[language][domain]["po"] = time.ctime(os.path.getmtime(po_file))

                pot_file = self._translation_template_path(translation_sources[domain])
                if os.path.isfile(pot_file):
                    translations[language][domain]["pot"] = time.ctime(os.path.getmtime(pot_file))

                mo_file = self._translation_file(language, domain, "mo")
                if os.path.isfile(mo_file):
                    translations[language][domain]["mo"] = time.ctime(os.path.getmtime(mo_file))

        return {"translations": translations}

    @view_config(route_name="localize:update")
    def update_catalogue(self):
        """Update or initialize translation catalogues.

        Create (.po files) for each language/catalogue from their respective
        translation templates (.pot). This action is performed for every language
        defined within `localize.locales.available` config key.

        Redirects itself to **localize:index**.
        """
        self.index()
        translation_sources = self.request.registry["localize"]["translation"]["sources"]

        for domain in translation_sources:
            pot_file = self._translation_template_path(translation_sources[domain])
            if not os.path.isfile(pot_file):
                log.critical("pot file for %s does not exists!", domain)

            for language in self.request.registry["localize"]["locales"]["available"]:
                po_file = self._translation_file(language, domain)
                if os.path.isfile(po_file):
                    log.debug(
                        "po file for %s, %s exists, proceeding with update",
                        domain,
                        language,
                    )

                    if subprocess.call(
                        [
                            self.request.registry["localize"]["pybabel"],
                            "update",
                            "-l",
                            language,
                            "-i",
                            pot_file,
                            "-o",
                            po_file,
                            "--previous",
                        ]
                    ):
                        log.error("Error while trying to update %s file!", po_file)
                else:
                    log.debug(
                        "po file for %s, %s does not exists, proceeding with initialize",
                        domain,
                        language,
                    )
                    if subprocess.call(
                        [
                            self.request.registry["localize"]["pybabel"],
                            "init",
                            "-l",
                            language,
                            "-i",
                            pot_file,
                            "-o",
                            po_file,
                        ]
                    ):
                        log.error("Error while trying to update %s file!", po_file)

        return HTTPFound(location=self.request.route_url("localize:index"))

    @view_config(route_name="localize:compile")
    def compile_catalogue(self):
        """Compile all translation files.

        For every language defined compile .po files into into .mo file that's used by gettext.

        Redirects to **localize:index**.
        """
        self.index()
        translation_sources = self.request.registry["localize"]["translation"]["sources"]

        for domain in translation_sources:
            for language in self.request.registry["localize"]["locales"]["available"]:
                po_file = self._translation_file(language, domain)
                mo_file = self._translation_file(language, domain, "mo")
                if os.path.isfile(po_file):
                    log.debug(
                        "po file for %s, %s exists, proceeding with compilation",
                        domain,
                        language,
                    )
                    if subprocess.call(
                        [
                            self.request.registry["localize"]["pybabel"],
                            "compile",
                            "-l",
                            language,
                            "-i",
                            po_file,
                            "-o",
                            mo_file,
                            "--use-fuzzy",
                        ]
                    ):
                        log.error("Error while trying to compile %s file!", po_file)

        return HTTPFound(location=self.request.route_url("localize:index"))

    @view_config(route_name="localize:reload", xhr="True", renderer="json")
    @view_config(route_name="localize:reload")
    def reload_catalogue(self):
        """Reload translation catalogue for application it's run in.

        .. note::

            To see how is this happening, you might want to see
            :func:`~pyramid_localize.tools.set_localizer`

        :returns:

            Only for xhr requests:

            .. code-block:: python

                {
                    'status': True,
                    'msg': 'Localizators has been reloaded' # translated
                }

            non xhr requests: Redirects to **localize:index**.


        """
        set_localizer(self.request, True)

        if self.request.is_xhr:
            return {
                "status": True,
                "msg": self.request._("Localizators has been reloaded"),
            }

        return HTTPFound(location=self.request.route_url("localize:index"))



"""Locale negotiator related tests."""

from pyramid_localize.negotiator import locale_negotiator


def test_negotiate_attr(locale_negotiator_request):
    """Locale_negotiator - negotiate locale from request attribute."""
    locale = locale_negotiator(locale_negotiator_request)

    assert locale == "fr"


def test_negotiate_path(locale_negotiator_request):
    """Locale_negotiator - negotiate locale from path."""
    locale_negotiator_request._LOCALE_ = None
    locale = locale_negotiator(locale_negotiator_request)

    assert locale == "pl"


def test_negotiate_cookie(locale_negotiator_request):
    """locale_negotiator - negotiate locale from cookie."""
    locale_negotiator_request._LOCALE_ = None
    locale_negotiator_request.path = "/page"
    locale = locale_negotiator(locale_negotiator_request)

    assert locale == "cz"


def test_negotiate_headers(locale_negotiator_request):
    """Locale_negotiator:header.

    Negotiate locale from a header.
    """
    locale_negotiator_request._LOCALE_ = None
    locale_negotiator_request.path = "/page"
    locale_negotiator_request.cookies = {}
    locale = locale_negotiator(locale_negotiator_request)

    assert locale == "de"


def test_negotiate_default(locale_negotiator_request):
    """Locale_negotiator:default.

    Other ways fail, return default locale.
    """
    locale_negotiator_request.path = "/page"
    locale_negotiator_request._LOCALE_ = None
    locale_negotiator_request.cookies = {}
    locale_negotiator_request.accept_language = None
    locale = locale_negotiator(locale_negotiator_request)

    assert locale == "en"



# Copyright (c) 2013-2014 by pyramid_localize authors and contributors <see AUTHORS file>
#
# This module is part of pyramid_localize and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""Locale negotiator."""


def locale_negotiator(request):
    """Locale negotiator.

    It sets best suited locale variable for given user:

    1. Check for presence and value of **request._LOCALE_** value
    2. Then tries the address url, if the first part has locale indicator.
    3. It checks cookies, for value set here
    4. Tries to best match accepted language for browser user is visiting
        website with
    5. Defaults to **localize.locales.default** configuration setting value

    :param pyramid.request.Request request: a request object
    :returns: locale name
    :rtype: str
    """
    available_languages = request.registry["localize"]["locales"]["available"]
    locale = request.registry["localize"]["locales"]["default"]
    # We do not have a matchdict present at the moment, lets get our own split
    # (request.path is always a /, so we'll get two elements)
    route_elements = request.path.split("/")
    if hasattr(request, "_LOCALE_") and request._LOCALE_ in available_languages:
        locale = request._LOCALE_
    # we check if route_element[1] is a locale indicator for path
    elif len(route_elements[1]) == 2 and route_elements[1] in available_languages:  # noqa: PLR2004
        locale = route_elements[1]
    elif request.cookies and "_LOCALE_" in request.cookies and request.cookies["_LOCALE_"] in available_languages:
        locale = request.cookies["_LOCALE_"]
    elif request.accept_language:
        locale = request.accept_language.best_match(available_languages)

    return locale



# Copyright (c) 2013-2014 by pyramid_localize authors and contributors <see AUTHORS file>
#
# This module is part of pyramid_localize and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""Subscribers adding mocked translation methods to render context, and request."""

from pyramid.events import BeforeRender, NewRequest, subscriber

from pyramid_localize.tools import dummy_autotranslate


@subscriber(BeforeRender)
def global_renderer(event):
    """Add fake localizer, and translation methods to context."""
    request = event["request"]
    try:
        event["_"] = request._
    except AttributeError:
        event["_"] = dummy_autotranslate


@subscriber(NewRequest)
def add_localizer(event):
    """Add fake localizer and translation methods to request."""
    event.request._ = dummy_autotranslate



# Copyright (c) 2013-2014 by pyramid_localize authors and contributors <see AUTHORS file>
#
# This module is part of pyramid_localize and is released under
# the MIT License (MIT): http://opensource.org/licenses/MIT
"""pyramid_localize configuration module."""

try:  # pragma: no cover
    import babel
except ImportError:  # pragma: no cover
    babel = False

from pyramid_localize.request import database_locales, locale_id, locales

__version__ = "1.0.7"


def build_localize_config(settings):
    """Build localize settings."""
    localize_config = {
        "pybabel": "pybabel",
        "locales": {  # available and default locale for your app
            "available": ["en", "de", "pl"],
            "default": "en",
        },
        "translation": {
            "dirs": [],
        },
    }
    localize_settings = {s: settings[s] for s in settings if s.startswith("localize")}
    for setting_key, setting_value in localize_settings.items():
        if setting_key == "localize.pybabel":
            localize_config["pybabel"] = setting_value
        elif setting_key == "localize.locales.available":
            localize_config["locales"]["available"] = setting_value
        elif setting_key == "localize.locales.default":
            localize_config["locales"]["default"] = setting_value
        elif setting_key == "localize.translation.dirs":
            localize_config["translation"]["dirs"] = setting_value
        elif setting_key == "localize.translation.destination":
            localize_config["translation"]["destination"] = setting_value
        elif setting_key == "localize.domain":
            localize_config["domain"] = setting_value

    return localize_config


def includeme(configurator):
    """pyramid_localize configuration method."""
    # let's check if we have any configuration, or not
    if babel:
        configurator.scan("pyramid_localize.subscribers.i18n")
        localize_config = build_localize_config(configurator.get_settings())
        configurator.registry["localize"] = localize_config
        configurator.include("pyramid_mako")

        configurator.set_locale_negotiator("pyramid_localize.negotiator.locale_negotiator")
        translation_dirs = localize_config["translation"]["dirs"]
        # if it's not a list, lets make it a list.
        # This is to allow creating both single, and list-like config entry
        if not isinstance(translation_dirs, list):
            translation_dirs = [translation_dirs]

        # let's add destination folder to the list
        if "destination" in localize_config["translation"]:
            translation_dirs.append(localize_config["translation"]["destination"])

        configurator.add_translation_dirs(*translation_dirs)
        # let scan all subscribers
        configurator.scan("pyramid_localize.views")

        configurator.add_route(name="localize:index", pattern="catalogue")
        configurator.add_route(name="localize:update", pattern="catalogue/update")
        configurator.add_route(name="localize:compile", pattern="catalogue/compile")
        configurator.add_route(name="localize:reload", pattern="catalogue/reload")

        # getting requests methods
        configurator.add_request_method(database_locales, name="_database_locales", reify=True)
        configurator.add_request_method(locales, name="locales")
        configurator.add_request_method(locale_id, name="locale_id", reify=True)

        # if configured
        return

    # including fake subscribers
    configurator.scan("pyramid_localize.subscribers.fake")



"""Subscribers related tests."""

from pyramid.events import BeforeRender, NewRequest
from pyramid.i18n import Localizer

# for this tests, these will be imported internally by pyramid's config
# from pyramid_localize.subscribers.i18n import global_renderer
# from pyramid_localize.subscribers.i18n import add_localizer
# from pyramid_localize.subscribers.fake import global_renderer
# from pyramid_localize.subscribers.fake import add_localizer


def test_i18n_new_request(request_i18n):
    """Test if method are being added to request."""
    request_i18n.registry.notify(NewRequest(request_i18n))
    assert isinstance(request_i18n.localizer, Localizer)
    assert hasattr(request_i18n, "_")


def test_i18n_before_render(request_i18n):
    """Test if appropriate methods are being added to render context."""
    before_render_event = BeforeRender({"request": request_i18n}, {})
    request_i18n.registry.notify(before_render_event)
    assert "localizer" in before_render_event
    assert "_" in before_render_event


def test_i18n_before_render_and_request(request_i18n):
    """Test if appropriate methods are being added to both context and request."""
    request_i18n.registry.notify(NewRequest(request_i18n))
    before_render_event = BeforeRender({"request": request_i18n}, {})
    request_i18n.registry.notify(before_render_event)
    assert "localizer" in before_render_event
    assert "_" in before_render_event


def test_fake_new_request(request_fake):
    """Test if method are being added to request."""
    request_fake.registry.notify(NewRequest(request_fake))
    assert hasattr(request_fake, "_")


def test_fake_before_render(request_fake):
    """Test if appropriate methods are being added to both context and request."""
    request_fake.registry.notify(NewRequest(request_fake))
    before_render_event = BeforeRender({"request": request_fake}, {})
    request_fake.registry.notify(before_render_event)
    assert "_" in before_render_event


def test_fake_before_render_norequest(request_fake):
    """Test if appropriate methods are being added to render context."""
    before_render_event = BeforeRender({"request": request_fake}, {})
    request_fake.registry.notify(before_render_event)
    assert "_" in before_render_event
